import json
import uuid

from xml_utils.xml_reader import EventSeverity


class Alarm:
    def __init__(self, logger, event_id):
        self.logger = logger
        self.id = event_id
        self.handle = str(uuid.uuid4())  # Generate a unique handle for the alarm

    def raise_alarm(self, text):
        if self.id not in self.logger.event_dictionary:
            print(f"invalid event_id:{self.id}")
            return

        if not self.logger.event_dictionary[self.id]["status"]:
            return  # Alarm is inactive

        event_object = self.logger.map_event(self.id, text)
        event_object["iid"] = (
            "coreiid://" + str(self.logger.iid) + "?raise=" + self.handle
        )
        # Write the event object to the stream
        self.logger.stream.write(json.dumps(event_object) + "\n")
        self.logger.alarms.add(self)  # Add the alarm to the set of active alarms

    def clear_alarm(self, text):
        if self.id not in self.logger.event_dictionary:
            print(f"invalid event_id:{self.id}")
            return

        if not self.logger.event_dictionary[self.id]["status"]:
            return  # Alarm is inactive

        event_object = self.logger.map_event(self.id, text)
        event_object["iid"] = (
            "coreiid://" + str(self.logger.iid) + "?clear=" + self.handle
        )
        # Set severity to 5 for clearing the alarm
        event_object["severity"] = EventSeverity.clear.value
        # Write the event object to the stream
        self.logger.stream.write(json.dumps(event_object) + "\n")
        self.logger.alarms.remove(
            self
        )  # Remove the alarm from the set of active alarms
