# import logging
import os
import socket
from functools import lru_cache

from dotenv import load_dotenv
from pydantic import BaseSettings

load_dotenv(verbose=True)


class KafkaSettings(BaseSettings):
    """
    Celery settings for worker threads
    """

    KAFKA_BROKERS: str = os.environ.get(
        "KAFKA_BROKERS", "host1:9092"
    )  # comma separated list of hosts - "host1:9092,host2:9092"


class Settings(KafkaSettings):
    OAM_VERSION: str = os.environ.get("OAM_VERSION", "2.2.0.0")
    APP_NAME: str = "PYTHON-CORE-OAM"
    # Directory of event_definition xml files
    EVENT_DEF_XML_DIR = "./event_definitions"
    HOST_INSTANCE_ID = os.environ.get(
        "HOST_INSTANCE_ID", "00000000-0000-0000-0000-000000000000"
    )
    DEBUG_MODE: bool = False
    HOSTNAME: str = socket.gethostname()
    WORKER_NODE_NAME: str = os.environ.get("WORKER_NODE_NAME", "")

    # NOTSET=0, DEBUG=10, INFO=20, WARN=30, ERROR=40, CRITICAL=50
    # LOG_LEVEL: int = logging.INFO  # info level logs req-resp json
    # in general set DEBUG for dev, WARN for production(least longing)

    class Config:
        env_file = ".env"  # override the defaults
        # env_file_encoding = 'utf-8'


# @lru_cache() modifies the function it decorates to return the same value that
# was returned the first time, instead of executing the code every time
@lru_cache()
def get_settings():
    return Settings()
