import datetime
import json

from alarm import Alarm
from protocols.text_out import TextOut
from utils import get_source
from xml_utils.xml_reader import EventType, build_event_dict


class EventLogger:
    def __init__(self, name, dir_path, stream: TextOut):
        # Initialize the Logger object with the provided parameters
        self.source = get_source(name)
        self.iid = 1
        self.stream = stream
        self.event_dictionary = build_event_dict(dir_path)
        self.alarms = set()
        # print(self.event_dictionary)

        # # TODO:Clear all alarms
        clearall = self.map_event("*", "clear all alarms")
        clearall["iid"] = f"coreiid://{self.iid}?clear=all"
        self.stream.write(json.dumps(clearall) + "\n")

    def log_event(self, event_id, message):
        if event_id not in self.event_dictionary:
            print(f"invalid event_id:{event_id}")
            return

        # TODO: don't log event for status == False?
        if not self.event_dictionary[event_id]["status"]:
            return

        # TODO: Cannot use for Alarm # 1:
        if self.event_dictionary[event_id]["eventType"] == EventType.Alarm.value:
            return
        # TODO: Just write to stdout for info (always?)
        elif self.event_dictionary[event_id]["eventType"] == EventType.Info.value:
            print(f"{datetime.datetime.now().isoformat()} {event_id}: {message}")
            return

        event_object = self.map_event(event_id, message)
        event_object["iid"] = f"coreiid://{self.iid}"
        self.stream.write(json.dumps(event_object) + "\n")

    def map_event(self, event_id, message):
        event = self.event_dictionary[event_id] if event_id != "*" else None

        category = 0
        severity = 0
        lifecycle = 0

        if event_id != "*":
            category = event["category"]
            severity = event["severity"]
            lifecycle = event["lifecycle"]

        msg = {
            "id": f"coreaction://{event_id}",
            "category": category,
            "date": datetime.datetime.now().isoformat(),
            "info": message,
            "tid": "00000000-0000-0000-0000-000000000000",
            "severity": severity,
            "source": self.source,
            "type": 1,
            "data": {"lifecycle": lifecycle},
        }
        return msg

    # def severity_to_str(self, severity):
    #     if severity == 1:
    #         return "WARN"
    #     elif severity == 2:
    #         return "MINOR"
    #     elif severity == 3:
    #         return "MAJOR"
    #     elif severity == 4:
    #         return "CRIT"
    #     elif severity == 5:
    #         return "CLEAR"
    #     return "WARN"

    def create_alarm(self, event_id):
        print(f"event_id:{event_id}")
        if event_id not in self.event_dictionary:
            print(f"invalid event_id:{event_id}")
            return
        if self.event_dictionary[event_id]["eventType"] != EventType.Alarm.value:
            print("not an alarm")
            return
        alarm = Alarm(logger=self, event_id=event_id)
        return alarm

    def shutdown(self):
        # Clear outstanding alarms
        for alarm in self.alarms:
            alarm.clear("Cleared by AlarmMgr")


if __name__ == "__main__":
    from sys import stdout

    from config import Settings, get_settings

    settings: Settings = get_settings()
    logger = EventLogger(
        name="SampleLogger",
        dir_path=settings.EVENT_DEF_XML_DIR,
        stream=stdout,
    )
