from typing import AnyStr

from config import Settings, get_settings

settings: Settings = get_settings()

from confluent_kafka import KafkaError, KafkaException, Producer


class SingletonKafkaProducer:
    """
    One kafka producer for all topics and partitions is optimal
    for most applications
    """

    producer: Producer = None

    @classmethod
    def get_producer(cls) -> Producer:
        try:
            if cls.producer is None:
                producer_conf = {
                    "bootstrap.servers": settings.KAFKA_BROKERS,
                    "client.id": settings.HOSTNAME,
                }
                cls.producer = Producer(producer_conf)
        except KafkaException as error:
            kafka_error = error.args[0]
            print(f"kafka_error:{kafka_error}")
            if kafka_error.code() == KafkaError._UNKNOWN_PARTITION:
                print("Kafka Topic/Partition Does Not Exist!!")
        return cls.producer


class KafkaTopicWriter:
    """implicitly considered to be a subtype of TextOut Protocol"""

    def __init__(self, topic: str) -> None:
        self.topic = topic
        # TODO: handle Kafka Connection errors - failure to get a producer
        self.producer = SingletonKafkaProducer.get_producer()

    def write(self, s: AnyStr):
        self.producer.produce(self.topic, s)
        # TODO: try async produce(), key, value, partitions
        self.producer.flush()  # makes the producer synchronous


if __name__ == "__main__":
    import json

    topic = "quickstart"
    topic_writer = KafkaTopicWriter(topic)

    event_object = {
        "date": "2023-06-01T13:28:02.698455",
        "start": "2023-06-01T13:28:01.698455",
        "duration": 1,
        "source": "coresource://ISE-LP-081.local/00000000-0000-0000-0000-000000000000/test/oamnode",
        "type": 4,
        "info": "OAM",
        "metrics": [
            {"name": "Foo.Bar", "value": 0, "metric_type": 1},
            {
                "name": "App.Metrics",
                "metricType": 2,
                "current_count": 0,
                "peak_count": 0,
                "total_count": 0,
                "failure_count": 0,
                "success_count": 0,
                "success_peak_time": 0,
                "failure_peak_time": 0,
                "success_total_time": 0,
                "failure_total_time": 0,
            },
        ],
    }
    topic_writer.write(json.dumps(event_object) + "\n")
