import json
from datetime import datetime, timedelta

from app_metrics import AppMetrics
from peg_count import PegCount
from protocols.text_out import TextOut
from utils import PeriodicTimer, get_source


class MetricsMgr:
    def __init__(
        self, name: str, stream: TextOut, report_interval: int = 0, om_info: str = "OAM"
    ):
        self.source = get_source(name)
        self.peg_counts = []
        self.app_metrics = []
        # self.timeout_id = 0
        # self.interval_id = 0
        now = datetime.now()
        self.start_time = now
        self.end_time = now + timedelta(seconds=60)

        if report_interval == 0:
            self.interval = 60  # defaults to 60 seconds
        else:
            self.interval = report_interval

        self.info = om_info
        self.stream = stream
        self.running = True
        self.schedule_periodic_reporting()

    def schedule_periodic_reporting(self):
        # print(".......schedule periodic reporting........")
        # print(f"start_time:{self.start_time}, end_time:{self.end_time}")
        # duration = self.end_time - self.start_time
        # timeout_seconds = duration.seconds
        # print(f"--------------timeout_seconds:{timeout_seconds}------------")
        # time.sleep(timeout_seconds)

        if not self.running:
            return
        print(f"periodic_reporting() is set up with interval - {self.interval}")
        self.timer = PeriodicTimer(self.interval, self.report)
        self.timer.start()

    def shutdown(self):
        self.running = False
        # print(f"timeout_id: {self.timeout_id}, interval_id: {self.interval_id}")
        # if self.timeout_id != 0:
        #     # clearTimeout(self.timeout_id)
        #     print(f"clearing up timeout_id:{self.timeout_id}")
        #     self.timeout_id = 0
        # print("---- shutting down the timer -----")
        self.timer.cancel()
        # if self.timer is not None:
        #     # clearInterval(self.interval_id)
        #     # print(f"clearing up timeout_id:{self.interval_id}")
        #     # self.interval_id = 0
        #     self.timer.cancel()

    def create_peg_count(self, name) -> PegCount:
        p = PegCount(name)
        self.peg_counts.append(p)
        return p

    def create_app_metrics(self, name):
        a = AppMetrics(name)
        self.app_metrics.append(a)
        return a

    def report(self):
        msg = {
            "date": self.end_time.isoformat(),
            "start": self.start_time.isoformat(),
            "duration": self.interval,
            "source": self.source,
            "type": 4,
            "info": self.info,
            "metrics": [],
        }

        for i in range(len(self.peg_counts)):
            print(self.peg_counts[i].__dict__)
            # msg["metrics"].append(self.peg_counts[i].to_dict())
            msg["metrics"].append(self.peg_counts[i].__dict__)

        for i in range(len(self.app_metrics)):
            print(self.app_metrics[i].__dict__)
            # msg["metrics"].append(self.app_metrics[i].to_dict())
            msg["metrics"].append(self.app_metrics[i].__dict__)

        self.stream.write(json.dumps(msg) + "\n")

        for i in range(len(self.peg_counts)):
            self.peg_counts[i].reset()

        for i in range(len(self.app_metrics)):
            self.app_metrics[i].reset()

        self.start_time = self.end_time
        self.end_time = self.start_time + timedelta(seconds=self.interval)
