import os
import time

# import threading
from threading import Timer

from config import Settings, get_settings

settings: Settings = get_settings()


class PeriodicTimer(Timer):
    """Repeats the execution of function passed to the Timer
       in periodic intervals

    Args:
        Timer (_type_): _description_
    """

    def run(self):
        # print(f"repeat_time interval:{self.interval}")
        while not self.finished.wait(self.interval):
            self.function(*self.args, **self.kwargs)
            # print(" ")


def set_interval(func, every_n_sec) -> Timer:
    """Recursively schedule the next execution of func


    Args:
        func (_type_): function to be executed
        every_n_sec (_type_): interval

    Returns:
        Timer:
    """

    def func_wrapper():
        set_interval(func, every_n_sec)
        func()  # Execute the function

    timer = Timer(every_n_sec, func_wrapper)  # Create a Timer object
    timer.start()  # Start the timer
    return timer  # Return the Timer object


def get_source(name: str):
    """
    Generate the source string based on the provided name and system env var
    """
    # Get the value of HOST_INSTANCE_ID environment variable,
    # or use a default value if it is not defined.
    # Construct the source string using hostname, host instance ID, and name.
    source = (
        f"coresource://{settings.HOSTNAME}/{settings.HOST_INSTANCE_ID}/{name}/oamnode"
    )
    # Check if the WORKER_NODE_NAME environment variable is defined.
    # If it is, append it to the source string.
    if settings.WORKER_NODE_NAME is not None:
        source += f"?worker={settings.WORKER_NODE_NAME}"

    return source


def display(msg):
    """tester function to check timer

    Args:
        msg (_type_): _description_
    """
    print(msg + " " + time.strftime("%H:%M:%S"))


if __name__ == "__main__":
    # #We are now creating a thread timer and controling it
    timer = PeriodicTimer(1, display, ["Repeating"])
    timer.start()  # recalling run
    print("Threading started")
    time.sleep(5)  # It gets suspended for the given number of seconds
    print("Threading finishing")
    timer.cancel()

    # event = set_interval(display, 2)
    # time.sleep(10)
    # # if timer.is_alive():
    # #     print("Timer is still alive")
    # # else:
    # #     print("Timer is no more alive")

    # # timer.cancel()/
    # print("cancelling the timer")
    # event.set()

    # name = "example"
    # print(get_source(name))
