import time

from config import Settings, get_settings

# from src.alarm import Alarm
from src.app_metrics import AppMetrics
from src.event_logger import EventLogger
from src.item import Item
from src.kafka_topic_writer import KafkaTopicWriter
from src.metrics_manager import MetricsMgr
from src.peg_count import PegCount
from src.protocols.text_out import TextOut

settings: Settings = get_settings()


def test_metrics_manager(stream: TextOut):
    # # # create metrics and increment
    # # report_interval defaults to 60 sec
    report_interval = 1
    metrics_manager: MetricsMgr = MetricsMgr("test", stream, report_interval)
    peg: PegCount = metrics_manager.create_peg_count("Foo.Bar")
    peg.increment()
    peg.increment()
    peg.increment()

    app_metrics: AppMetrics = metrics_manager.create_app_metrics("App.Metrics")
    item = Item(app_metrics=app_metrics)
    item.start()
    item.stop()

    # background reporting keeps this running. shutdown after some time.
    shutdown_in_sec = 10
    print(f"shutting the metrics_manager down in {shutdown_in_sec} sec")
    time.sleep(shutdown_in_sec)  # sleep for 10 sec

    metrics_manager.shutdown()


def test_logger_events(stream: TextOut):
    # test logging events
    event_logger = EventLogger(
        "testapp",
        settings.EVENT_DEF_XML_DIR,
        stream,
    )  # read events xmls in this "test" directory
    # log fault
    event_logger.log_event("CORE-Err_UnreachableCode", "Hello World!")
    # log info
    event_logger.log_event("CORE-CSD_PrimaryTime_PrimaryChanged", "Test Info")

    alarm = event_logger.create_alarm(
        event_id="CORE-Kafka_Overload"
    )  # raise and clear an alarm
    if alarm is not None:
        alarm.raise_alarm("foo")
        alarm.clear_alarm("bar")


if __name__ == "__main__":
    # console #################
    # TODO: fix No overloaded function matches type "(s: AnyStr@write) -> None"
    # from sys import stdout

    # stream = stdout
    ####################
    # # Kafka ###################
    topic = "quickstart"
    topic_writer = KafkaTopicWriter(topic)
    stream = topic_writer
    ####################

    test_logger_events(stream)
    print("---------------------------------------------------------------")
    test_metrics_manager(stream)
    print("---------------------------------------------------------------")
